@@ -1,9 +1,13 @@
+from __future__ import annotations
+
+import enum
 import os
 from itertools import chain
 
 import pytest
 
 import asyncclick as click
+from asyncclick._utils import UNSET
 
 
 def test_basic_functionality(runner):
@@ -204,34 +208,52 @@ def test_float_option(runner, args, expect):
         assert result.exception is None
 
 
-@pytest.mark.parametrize("default", [True, False])
 @pytest.mark.parametrize(
-    ("args", "expect"), [(["--on"], True), (["--off"], False), ([], None)]
+    ("args", "default", "expect"),
+    [
+        (["--on"], True, True),
+        (["--on"], False, True),
+        (["--on"], None, True),
+        (["--on"], UNSET, True),
+        (["--off"], True, False),
+        (["--off"], False, False),
+        (["--off"], None, False),
+        (["--off"], UNSET, False),
+        ([], True, True),
+        ([], False, False),
+        ([], None, None),
+        ([], UNSET, False),
+    ],
 )
-def test_boolean_switch(runner, default, args, expect):
+def test_boolean_switch(runner, args, default, expect):
     @click.command()
     @click.option("--on/--off", default=default)
     def cli(on):
         return on
 
-    if expect is None:
-        expect = default
-
     result = runner.invoke(cli, args, standalone_mode=False)
     assert result.return_value is expect
 
 
-@pytest.mark.parametrize("default", [True, False])
-@pytest.mark.parametrize(("args", "expect"), [(["--f"], True), ([], False)])
+@pytest.mark.parametrize(
+    ("default", "args", "expect"),
+    (
+        (True, ["--f"], True),
+        (True, [], True),
+        (False, ["--f"], True),
+        (False, [], False),
+        # Boolean flags have a 3-states logic.
+        # See: https://github.com/pallets/click/issues/3024#issue-3285556668
+        (None, ["--f"], True),
+        (None, [], None),
+    ),
+)
 def test_boolean_flag(runner, default, args, expect):
     @click.command()
     @click.option("--f", is_flag=True, default=default)
     def cli(f):
         return f
 
-    if default:
-        expect = not expect
-
     result = runner.invoke(cli, args, standalone_mode=False)
     assert result.return_value is expect
 
@@ -256,6 +278,58 @@ def test_boolean_conversion(runner, value, expect):
     assert result.output == expect
 
 
+@pytest.mark.parametrize(
+    ("default", "args", "expected"),
+    # These test cases are similar to the ones in
+    # tests/test_options.py::test_default_dual_option_callback, so keep them in sync.
+    (
+        # Each option is returning its own flag_value, whatever the default is.
+        (True, ["--upper"], "upper"),
+        (True, ["--lower"], "lower"),
+        (False, ["--upper"], "upper"),
+        (False, ["--lower"], "lower"),
+        (None, ["--upper"], "upper"),
+        (None, ["--lower"], "lower"),
+        (UNSET, ["--upper"], "upper"),
+        (UNSET, ["--lower"], "lower"),
+        # Check that the last option wins when both are specified.
+        (True, ["--upper", "--lower"], "lower"),
+        (True, ["--lower", "--upper"], "upper"),
+        # Check that the default is returned as-is when no option is specified.
+        ("upper", [], "upper"),
+        ("lower", [], "lower"),
+        ("uPPer", [], "uPPer"),
+        ("lOwEr", [], "lOwEr"),
+        (" ᕕ( ᐛ )ᕗ ", [], " ᕕ( ᐛ )ᕗ "),
+        (None, [], None),
+        # Default is normalized to None if it is UNSET.
+        (UNSET, [], None),
+        # Special case: if default=True and flag_value is set, the value returned is the
+        # flag_value, not the True Python value itself.
+        (True, [], "upper"),
+        # Non-string defaults are process as strings by the default Parameter's type.
+        (False, [], "False"),
+        (42, [], "42"),
+        (12.3, [], "12.3"),
+    ),
+)
+def test_flag_value_dual_options(runner, default, args, expected):
+    """Check how default is processed when options compete for the same variable name.
+
+    Covers the regression reported in
+    https://github.com/pallets/click/issues/3024#issuecomment-3146199461
+    """
+
+    @click.command()
+    @click.option("--upper", "case", flag_value="upper", default=default)
+    @click.option("--lower", "case", flag_value="lower")
+    def cli(case):
+        click.echo(repr(case), nl=False)
+
+    result = runner.invoke(cli, args)
+    assert result.output == repr(expected)
+
+
 def test_file_option(runner):
     @click.command()
     @click.option("--file", type=click.File("w"))
@@ -405,6 +479,92 @@ def test_choice_argument(runner):
     assert "{foo|bar|baz}" in result.output
 
 
+def test_choice_argument_enum(runner):
+    class MyEnum(str, enum.Enum):
+        FOO = "foo-value"
+        BAR = "bar-value"
+        BAZ = "baz-value"
+
+    @click.command()
+    @click.argument("method", type=click.Choice(MyEnum, case_sensitive=False))
+    def cli(method: MyEnum):
+        assert isinstance(method, MyEnum)
+        click.echo(method)
+
+    result = runner.invoke(cli, ["foo"])
+    assert result.output == "foo-value\n"
+    assert not result.exception
+
+    result = runner.invoke(cli, ["meh"])
+    assert result.exit_code == 2
+    assert (
+        "Invalid value for '{foo|bar|baz}': 'meh' is not one of 'foo',"
+        " 'bar', 'baz'." in result.output
+    )
+
+    result = runner.invoke(cli, ["--help"])
+    assert "{foo|bar|baz}" in result.output
+
+
+def test_choice_argument_custom_type(runner):
+    class MyClass:
+        def __init__(self, value: str) -> None:
+            self.value = value
+
+        def __str__(self) -> str:
+            return self.value
+
+    @click.command()
+    @click.argument(
+        "method", type=click.Choice([MyClass("foo"), MyClass("bar"), MyClass("baz")])
+    )
+    def cli(method: MyClass):
+        assert isinstance(method, MyClass)
+        click.echo(method)
+
+    result = runner.invoke(cli, ["foo"])
+    assert not result.exception
+    assert result.output == "foo\n"
+
+    result = runner.invoke(cli, ["meh"])
+    assert result.exit_code == 2
+    assert (
+        "Invalid value for '{foo|bar|baz}': 'meh' is not one of 'foo',"
+        " 'bar', 'baz'." in result.output
+    )
+
+    result = runner.invoke(cli, ["--help"])
+    assert "{foo|bar|baz}" in result.output
+
+
+def test_choice_argument_none(runner):
+    @click.command()
+    @click.argument(
+        "method", type=click.Choice(["not-none", None], case_sensitive=False)
+    )
+    def cli(method: str | None):
+        assert isinstance(method, str) or method is None
+        click.echo(repr(method), nl=False)
+
+    result = runner.invoke(cli, ["not-none"])
+    assert not result.exception
+    assert result.output == repr("not-none")
+
+    result = runner.invoke(cli, ["none"])
+    assert not result.exception
+    assert result.output == repr(None)
+
+    result = runner.invoke(cli, [])
+    assert result.exception
+    assert (
+        "Error: Missing argument '{not-none|none}'. "
+        "Choose from:\n\tnot-none,\n\tnone\n" in result.stderr
+    )
+
+    result = runner.invoke(cli, ["--help"])
+    assert result.output.startswith("Usage: cli [OPTIONS] {not-none|none}\n")
+
+
 def test_datetime_option_default(runner):
     @click.command()
     @click.option("--start_date", type=click.DateTime())
