@@ -205,6 +205,29 @@ async def test_catch_exceptions():
     assert result.exit_code == 1
 
 
+@pytest.mark.anyio
+async def test_catch_exceptions_cli_runner():
+    """Test that invoke `catch_exceptions` takes the value from CliRunner if not set
+    explicitly."""
+
+    class CustomError(Exception):
+        pass
+
+    @click.command()
+    async def cli():
+        raise CustomError(1)
+
+    runner = CliRunner(catch_exceptions=False)
+
+    result = await runner.invoke(cli, catch_exceptions=True)
+    assert isinstance(result.exception, CustomError)
+    assert type(result.exc_info) is tuple
+    assert len(result.exc_info) == 3
+
+    with pytest.raises(CustomError):
+        await runner.invoke(cli)
+
+
 @pytest.mark.anyio
 async def test_with_color():
     @click.command()
@@ -268,9 +291,9 @@ async def test_exit_code_and_output_from_sys_exit():
 
     @click.command()
     @click.pass_context
-    def cli_string_ctx_exit(ctx):
+    async def cli_string_ctx_exit(ctx):
         click.echo("hello world")
-        ctx.exit("error")
+        await ctx.aexit("error")
 
     @click.command()
     def cli_int():
@@ -279,9 +302,9 @@ async def test_exit_code_and_output_from_sys_exit():
 
     @click.command()
     @click.pass_context
-    def cli_int_ctx_exit(ctx):
+    async def cli_int_ctx_exit(ctx):
         click.echo("hello world")
-        ctx.exit(1)
+        await ctx.aexit(1)
 
     @click.command()
     def cli_float():
@@ -290,9 +313,9 @@ async def test_exit_code_and_output_from_sys_exit():
 
     @click.command()
     @click.pass_context
-    def cli_float_ctx_exit(ctx):
+    async def cli_float_ctx_exit(ctx):
         click.echo("hello world")
-        ctx.exit(1.0)
+        await ctx.aexit(1.0)
 
     @click.command()
     def cli_no_error():
@@ -352,32 +375,23 @@ async def test_env():
 async def test_stderr():
     @click.command()
     def cli_stderr():
-        click.echo("stdout")
-        click.echo("stderr", err=True)
+        click.echo("1 - stdout")
+        click.echo("2 - stderr", err=True)
+        click.echo("3 - stdout")
+        click.echo("4 - stderr", err=True)
 
-    runner = CliRunner(mix_stderr=False)
-
-    result = await runner.invoke(cli_stderr)
-
-    assert result.output == "stdout\n"
-    assert result.stdout == "stdout\n"
-    assert result.stderr == "stderr\n"
-
-    runner_mix = CliRunner(mix_stderr=True)
+    runner_mix = CliRunner()
     result_mix = await runner_mix.invoke(cli_stderr)
 
-    assert result_mix.output == "stdout\nstderr\n"
-    assert result_mix.stdout == "stdout\nstderr\n"
-
-    with pytest.raises(ValueError):
-        result_mix.stderr  # noqa B018
+    assert result_mix.output == "1 - stdout\n2 - stderr\n3 - stdout\n4 - stderr\n"
+    assert result_mix.stdout == "1 - stdout\n3 - stdout\n"
+    assert result_mix.stderr == "2 - stderr\n4 - stderr\n"
 
     @click.command()
     def cli_empty_stderr():
         click.echo("stdout")
 
-    runner = CliRunner(mix_stderr=False)
-
+    runner = CliRunner()
     result = await runner.invoke(cli_empty_stderr)
 
     assert result.output == "stdout\n"
@@ -466,9 +480,29 @@ def test_isolation_stderr_errors():
     """Writing to stderr should escape invalid characters instead of
     raising a UnicodeEncodeError.
     """
-    runner = CliRunner(mix_stderr=False)
+    runner = CliRunner()
 
-    with runner.isolation() as (_, err):
+    with runner.isolation() as (_, err, _):
         click.echo("\udce2", err=True, nl=False)
+        assert err.getvalue() == b"\\udce2"
 
-    assert err.getvalue() == b"\\udce2"
+
+@pytest.mark.anyio
+async def test_isolation_flushes_unflushed_stderr():
+    """An un-flushed write to stderr, as with `print(..., file=sys.stderr)`, will end up
+    flushed by the runner at end of invocation.
+    """
+    runner = CliRunner()
+
+    with runner.isolation() as (_, err, _):
+        click.echo("\udce2", err=True, nl=False)
+        assert err.getvalue() == b"\\udce2"
+
+    @click.command()
+    def cli():
+        # set end="", flush=False so that it's totally clear that we won't get any
+        # auto-flush behaviors
+        print("gyarados gyarados gyarados", file=sys.stderr, end="", flush=False)
+
+    result = await runner.invoke(cli)
+    assert result.stderr == "gyarados gyarados gyarados"
