@@ -1,10 +1,10 @@
-"""Example from https://click.palletsprojects.com/en/8.1.x/advanced/#command-aliases"""
+"""Example from https://click.palletsprojects.com/en/stable/advanced/#command-aliases"""
 
 from __future__ import annotations
 
 from typing_extensions import assert_type
 
-import click
+import asyncclick as click
 
 
 class AliasedGroup(click.Group):
@@ -19,11 +19,11 @@ class AliasedGroup(click.Group):
             return click.Group.get_command(self, ctx, matches[0])
         ctx.fail(f"Too many matches: {', '.join(sorted(matches))}")
 
-    def resolve_command(
+    async def resolve_command(
         self, ctx: click.Context, args: list[str]
     ) -> tuple[str | None, click.Command, list[str]]:
         # always return the full command name
-        _, cmd, args = super().resolve_command(ctx, args)
+        _, cmd, args = await super().resolve_command(ctx, args)
         assert cmd is not None
         return cmd.name, cmd, args
 
