@@ -1,6 +1,6 @@
 from typing_extensions import assert_type
 
-import click
+import asyncclick as click
 
 
 @click.command()
