@@ -1,10 +1,10 @@
 """
-From https://click.palletsprojects.com/en/8.1.x/options/#callbacks-and-eager-options.
+From https://click.palletsprojects.com/en/stable/options/#callbacks-and-eager-options.
 """
 
 from typing_extensions import assert_type
 
-import click
+import asyncclick as click
 
 
 @click.command()
